from fennil.app.deck.faults import fault_line_layers

# Keep base fault colors stable and distinct per dataset.
FAULT_LINE_COLORS = [
    (0, 0, 255, 255),
    (0, 128, 0, 255),
]
FAULT_LINE_WIDTHS = (1, 1)


def build_fault_lines(ctx):
    for idx, dataset in enumerate(ctx.datasets):
        if not dataset.enabled or dataset.data is None:
            continue

        folder_number = idx + 1
        seg_tooltip_enabled = not idx
        fault_layers, _ = fault_line_layers(
            folder_number,
            dataset.data.segment,
            seg_tooltip_enabled,
            FAULT_LINE_COLORS[idx],
            FAULT_LINE_WIDTHS[idx],
        )
        ctx.layers.extend(fault_layers)



import numpy as np

VELOCITY_SCALE = 1000
VECTOR_ARROW_SIZE_FACTOR = 6
VECTOR_ARROW_MIN_PIXELS = 8
VECTOR_ARROW_MAX_PIXELS = 28

RES_COMPARE_SIZE_SCALE = VELOCITY_SCALE / 2500.0
RES_COMPARE_DIFF_MIN = -5.0
RES_COMPARE_DIFF_MAX = 5.0
RES_COMPARE_UNIQUE_SIZE_PIXELS = 15.0
RES_COMPARE_UNIQUE_COLOR = [0, 0, 0, 220]

FAULT_PROJ_LINE_WIDTH = 1
SLIP_WIDTH_SCALE = 0.05
SLIP_WIDTH_MIN_PIXELS = 1
SLIP_WIDTH_CAP_MM_PER_YR = 400.0
SLIP_NEGATIVE_COLOR = [31, 119, 180, 220]  # tab:blue
SLIP_POSITIVE_COLOR = [255, 127, 14, 220]  # tab:orange
SLIP_NEGATIVE_EXTREME_COLOR = [25, 230, 255, 220]  # bright blue
SLIP_POSITIVE_EXTREME_COLOR = [255, 40, 40, 220]  # bright red


# ColorBrewer RdBu[11] palette for discrete slip-rate coloring
RDBU_11 = [
    (103, 0, 31),
    (178, 24, 43),
    (214, 96, 77),
    (244, 165, 130),
    (253, 219, 199),
    (247, 247, 247),
    (209, 229, 240),
    (146, 197, 222),
    (67, 147, 195),
    (33, 102, 172),
    (5, 48, 97),
]
RED = [255, 0, 0, 255]
BLACK = [0, 0, 0, 255]

SLIP_RATE_MIN = -100.0
SLIP_RATE_MAX = 100.0


def map_slip_colors(values):
    """Map slip values to discrete RdBu[11] colors."""
    colors_array = []
    span = SLIP_RATE_MAX - SLIP_RATE_MIN
    for raw_value in values:
        value = raw_value
        if not np.isfinite(value):
            value = 0.0
        value = float(np.clip(value, SLIP_RATE_MIN, SLIP_RATE_MAX))
        position = (value - SLIP_RATE_MIN) / span
        index = int(np.floor(position * len(RDBU_11)))
        index = max(0, min(len(RDBU_11) - 1, index))
        r, g, b = RDBU_11[index]
        colors_array.append([r, g, b, 255])
    return colors_array



from fennil.app.deck.faults import fault_projection_layers
from fennil.app.io import Dataset
from fennil.app.registry import FieldSpec, LayerContext

SPEC = FieldSpec(
    priority=30,
    label="Fault Proj",
    icon="mdi-bandage",
    ui_type="VCheckbox",
    options=None,
    default=False,
    styles={
        "colors": [
            (128, 128, 128, 255),
            (128, 128, 128, 255),
        ],
        "line_width": (1, 2),
        "fill": [
            (173, 216, 230, 77),
            (144, 238, 144, 77),
        ],
        "line": [
            (0, 0, 255, 255),
            (0, 128, 0, 255),
        ],
    },
)


def builder(name: str, ctx: LayerContext):
    if ctx.skip(name):
        return

    for idx, dataset in ctx.enabled_datasets(name):
        ctx.layers.extend(
            fault_projection_layers(
                dataset.name,
                dataset.data.fault_proj_df,
                ctx.specs[name]["styles"]["fill"][idx],
                ctx.specs[name]["styles"]["line"][idx],
            )
        )


def can_render(dataset: Dataset) -> bool:
    if dataset is None:
        return False

    return dataset.fault_proj_available



from fennil.app.deck.tde import tde_mesh_layers, tde_perimeter_layers
from fennil.app.io import Dataset
from fennil.app.registry import FieldSpec, LayerContext

SPEC = FieldSpec(
    priority=21,
    label="TDE",
    icon="mdi-texture-box",
    ui_type="VBtnToggle",
    options=[
        {"text": "SS", "value": "ss"},
        {"text": "DS", "value": "ds"},
    ],
    default=None,
    styles={
        "icon_color": "rgba(14, 0, 214, 1)",
    },
)


def builder(name: str, ctx: LayerContext):
    if ctx.skip(name):
        return

    for idx, dataset in ctx.enabled_datasets(name):
        folder_number = idx + 1
        value = dataset.fields[name]
        tde_df = dataset.data.tde_df
        tde_perim_df = dataset.data.tde_perim_df
        if tde_df is not None and not tde_df.empty:
            slip_values = (
                tde_df["ss_rate"].to_numpy()
                if value == "ss"
                else tde_df["ds_rate"].to_numpy()
            )
            ctx.tde_layers.extend(tde_mesh_layers(folder_number, tde_df, slip_values))
        ctx.tde_layers.extend(tde_perimeter_layers(folder_number, tde_perim_df))


def can_render(dataset: Dataset) -> bool:
    return dataset is not None and dataset.tde_available



from fennil.app.deck.res_compare import residual_compare_layers
from fennil.app.io import Dataset
from fennil.app.registry import FieldSpec, LayerContext

SPEC = FieldSpec(
    priority=50,
    label="Res compare",
    icon="mdi-vector-difference",
    ui_type="VCheckbox",
    options=None,
    default=False,
    styles={
        "icon_color": "rgba(205, 0, 205, 0.78)",
        "colors": [
            (205, 0, 205, 200),
            (205, 0, 205, 200),
        ],
        "line_width": (1, 2),
    },
    multiple=False,
)


def builder(name: str, ctx: LayerContext):
    if ctx.skip(name):
        return

    right = ctx.datasets[0]
    left = ctx.datasets[1]
    if not (right.enabled and left.enabled):
        return
    if right.data is None or left.data is None:
        return

    ctx.layers.extend(
        residual_compare_layers(
            right.data,
            left.data,
            ctx.velocity_scale,
        )
    )


def can_render(dataset: Dataset) -> bool:
    return dataset is not None
